import {
  Grid,
  Paper,
  Tab,
  Tabs,
  Typography,
  IconButton
} from "@material-ui/core";
import ArrowBackIcon from "@material-ui/icons/ArrowBack";
import React, { useCallback, useEffect, useState } from "react";
import { useOngoingCommand, useCommandList } from "../../api/SwaggerApi";
import { IOCLiveStatus } from "../../components/IOC/IOCLiveStatus";
import { IOCManage } from "../../components/IOC/IOCManage";
import { useNavigate } from "react-router-dom";
import IOCAdmin from "../../components/IOC/IOCAdmin";
import AccessControl from "../../components/auth/AccessControl";
import { initRequestParams } from "../../components/common/Helper";
import { useGlobalAppBar } from "../../components/navigation/GlobalAppBar/GlobalAppBar";
import { useSafePolling } from "../../hooks/Polling";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";

const IOC_POLL_INTERVAL = 10000;
export function IOCDetailsView({ ioc, getIOC, loading }) {
  const [state, setState] = useUrlState({
    tab: "Status",
    commands_rows: "5",
    commands_page: "0"
  });

  const [buttonDisabled, setButtonDisabled] = useState(false);
  const navigate = useNavigate();
  const [commands, getCommands /* reset*/, , commandsLoading] =
    useCommandList();
  const [
    ongoingCommand,
    getOngoingCommand /* reset*/,
    ,
    ongoingCommandLoading
  ] = useOngoingCommand(ioc.id);

  const commandLazyParams = useCallback(() => {
    return {
      rows: deserialize(state.commands_rows),
      page: deserialize(state.commands_page)
    };
  }, [state]);

  const setCommandLazyParams = useCallback(
    (params) => {
      setState({
        commands_rows: serialize(params.rows),
        commands_page: serialize(params.page)
      });
    },
    [setState]
  );

  const [commandColumnSort, setCommandColumnSort] = useState({
    sortField: null,
    sortOrder: null
  });

  useSafePolling(getIOC, loading, IOC_POLL_INTERVAL);
  useSafePolling(getOngoingCommand, ongoingCommandLoading, IOC_POLL_INTERVAL);

  useEffect(() => {
    setButtonDisabled(Boolean(ioc?.operationInProgress));
  }, [ioc?.operationInProgress]);

  const handleTabChange = (event, tab) => {
    setState({ tab: serialize(tab) });
  };

  useEffect(() => {
    if (ioc.activeDeployment) {
      let requestParams = initRequestParams(
        commandLazyParams(),
        null,
        commandColumnSort
      );
      requestParams.ioc_id = ioc.id;
      // requestParams.deployment_id = ioc.activeDeployment.id;
      getCommands(requestParams);
    }
  }, [getCommands, commandLazyParams, commandColumnSort, ioc]);

  const handleClick = () => {
    navigate(-1);
  };

  const resetTab = useCallback(() => {
    if (ioc?.activeDeployment) {
      setState({ tab: "Status" });
    } else {
      setState({ tab: "Management" });
    }
  }, [ioc?.activeDeployment, setState]);

  const setButtonDisabledAndUpdate = useCallback(
    (isDisabled) => {
      setButtonDisabled(isDisabled);
      getIOC();
    },
    [getIOC]
  );

  const { setTitle } = useGlobalAppBar("IOC Details");

  useEffect(() => {
    ioc && setTitle("IOC Details: " + ioc.namingName);
  }, [ioc, setTitle]);

  const statusTab = (
    <Tab
      key="Status"
      label={<Typography variant="h5">Status</Typography>}
      value="Status"
    />
  );
  const manageDeploymentTab = (
    <Tab
      key="Management"
      label={<Typography variant="h5">Management</Typography>}
      value="Management"
    />
  );
  const iocAdminTab = (
    <Tab
      key="Admin"
      label={<Typography variant="h5">Admin</Typography>}
      value="Admin"
    />
  );

  const CustomTabs = ({ children }) => {
    return (
      <Tabs
        value={deserialize(state.tab)}
        onChange={handleTabChange}
        indicatorColor="primary"
        textColor="primary"
      >
        {children}
      </Tabs>
    );
  };

  return (
    <Paper>
      <Grid
        container
        spacing={1}
      >
        <Grid
          item
          xs={1}
        >
          <IconButton
            color="inherit"
            onClick={handleClick}
          >
            <ArrowBackIcon />
          </IconButton>
        </Grid>
        <Grid
          item
          xs={11}
        >
          <Grid
            container
            justify="center"
          >
            {ioc && (
              <CustomTabs>
                {statusTab}
                <AccessControl
                  allowedRoles={[
                    "DeploymentToolAdmin",
                    "DeploymentToolIntegrator"
                  ]}
                  renderNoAccess={() => <></>}
                >
                  <CustomTabs>{manageDeploymentTab}</CustomTabs>
                </AccessControl>
                <AccessControl
                  allowedRoles={[
                    "DeploymentToolAdmin",
                    "DeploymentToolManager"
                  ]}
                  allowedUsersWithRoles={[
                    { user: ioc.owner, role: "DeploymentToolIntegrator" }
                  ]}
                  renderNoAccess={() => <></>}
                >
                  <CustomTabs>{iocAdminTab}</CustomTabs>
                </AccessControl>
              </CustomTabs>
            )}
          </Grid>
        </Grid>
        <Grid
          item
          xs={12}
          style={{ paddingBottom: 0 }}
        >
          {deserialize(state.tab) === "Status" && <IOCLiveStatus ioc={ioc} />}
          {deserialize(state.tab) === "Management" && (
            <IOCManage
              ioc={ioc}
              getIOC={getIOC}
              buttonDisabled={buttonDisabled}
              currentCommand={ongoingCommand}
              commands={commands}
              commandsLoading={commandsLoading}
              getCommands={getCommands}
              setButtonDisabled={setButtonDisabledAndUpdate}
              commandColumnSort={commandColumnSort}
              setCommandColumnSort={setCommandColumnSort}
              commandLazyParams={commandLazyParams()}
              setCommandLazyParams={setCommandLazyParams}
            />
          )}
          {deserialize(state.tab) === "Admin" && (
            <IOCAdmin
              ioc={ioc}
              getIOC={getIOC}
              resetTab={resetTab}
              buttonDisabled={buttonDisabled}
            />
          )}
        </Grid>
      </Grid>
    </Paper>
  );
}
