import React, { useContext, useState } from "react";
import { makeStyles } from "@material-ui/core/styles";
import {
  Paper,
  Grid,
  FormControlLabel,
  Switch,
  Typography,
  Container
} from "@material-ui/core";
import Tabs from "@material-ui/core/Tabs";
import Tab from "@material-ui/core/Tab";
import { DeploymentAsyncList } from "../../components/deployments/DeploymentAsyncList";
import { useDeploymentSearch, userContext } from "../../api/SwaggerApi";
import { initRequestParams } from "../../components/common/Helper";
import { useEffect } from "react";
import { SearchBar } from "../../components/common/SearchBar/SearchBar";
import AccessControl from "../../components/auth/AccessControl";

const useStyles = makeStyles((theme) => ({
  root: {
    width: "100%"
  },
  paper: {
    marginBottom: theme.spacing(2)
  },
  formControl: {
    marginLeft: "5px",
    marginRight: "5px"
  }
}));

export function DeploymentsView() {
  const classes = useStyles();
  const [deployments, getDeployments /* reset*/, , loading] =
    useDeploymentSearch();
  const [query, setQuery] = useState("");
  const [selectedTab, setSelectedTab] = useState(0);
  const [ownOnly, setOwnOnly] = useState(false);
  const { user } = useContext(userContext);
  const [deploymentStatus, setDeploymentStatus] = useState(null);
  const [deploymentList, setDeploymentList] = useState([]);

  const handleTabChange = (event, tab) => {
    setSelectedTab(tab);

    if (tab === 0) {
      setDeploymentStatus(null);
    } else if (tab === 1) {
      setDeploymentStatus("RUNNING");
    } else if (tab === 2) {
      setDeploymentStatus("FINISHED");
    } else if (tab === 3) {
      setDeploymentStatus("QUEUED");
    }
  };

  const handleChangeOwn = (event) => {
    const own = event.target.checked;
    setOwnOnly(own);
  };

  const [lazyParams, setLazyParams] = useState({
    rows: 20,
    page: 0
  });

  const [columnSort, setColumnSort] = useState({
    sortField: null,
    sortOrder: null
  });

  const rowsPerPage = [20, 50];

  useEffect(() => {
    let requestParams = initRequestParams(lazyParams, query, columnSort);

    if (ownOnly) {
      requestParams.user = user?.loginName;
    }

    if (columnSort.sortField === "start") {
      requestParams.order_by = "TIME";
    }

    if (columnSort.sortField === "user") {
      requestParams.order_by = "CREATED_BY";
    }

    if (columnSort.sortField === "ioc") {
      requestParams.order_by = "IOC_NAME";
    }

    if (columnSort.sortField === "version") {
      requestParams.order_by = "GIT_REFERENCE";
    }

    if (deploymentStatus) {
      requestParams.status = deploymentStatus;
    }

    getDeployments(requestParams);
  }, [
    getDeployments,
    lazyParams,
    query,
    columnSort,
    user,
    deploymentStatus,
    ownOnly
  ]);

  useEffect(() => {
    setDeploymentList(deployments.deploymentList);
  }, [deployments, setDeploymentList]);

  const content = (
    <SearchBar
      search={setQuery}
      loading={loading}
    >
      <DeploymentAsyncList
        deployments={deploymentList}
        setDeployments={setDeploymentList}
        loading={loading}
        totalCount={deployments.totalCount}
        lazyParams={lazyParams}
        setLazyParams={setLazyParams}
        columnSort={columnSort}
        setColumnSort={setColumnSort}
        rowsPerPage={rowsPerPage}
      />
    </SearchBar>
  );

  return (
    <Paper className={classes.paper}>
      <Grid
        container
        spacing={1}
      >
        <Container>
          <Grid
            container
            spacing={1}
            justify="space-between"
          >
            <Grid
              item
              xs={8}
            >
              <Tabs
                value={selectedTab}
                onChange={handleTabChange}
                indicatorColor="primary"
                textColor="primary"
              >
                <Tab label={<Typography variant="h5">All</Typography>} />
                <Tab
                  label={<Typography variant="h5">Only Running</Typography>}
                />
                <Tab
                  label={<Typography variant="h5">Only Finished</Typography>}
                />
                <Tab
                  label={<Typography variant="h5">Only Queued</Typography>}
                />
              </Tabs>
            </Grid>
            <Grid item>
              <AccessControl
                allowedRoles={[
                  "DeploymentToolAdmin",
                  "DeploymentToolIntegrator"
                ]}
                renderNoAccess={() => <></>}
              >
                <FormControlLabel
                  className={classes.formControl}
                  control={
                    <Switch
                      checked={ownOnly}
                      onChange={handleChangeOwn}
                    />
                  }
                  label={
                    <Typography variant="h5">Only my deployments</Typography>
                  }
                />
              </AccessControl>
            </Grid>
          </Grid>
        </Container>
        <Grid
          item
          xs={12}
          md={12}
        >
          {content}
        </Grid>
      </Grid>
    </Paper>
  );
}
